
/*
Copyright 2007 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function SwitchDialog()
{
	this.sw = null; // switch
	this.st = null; // status
	this.pr = null; // preference
	this.index = -1; // selected table index

	this.control = null;
	this.listboxSwitches = null;
	this.dialogheaderSwitch = null;
	this.initial = null;
	this.listboxStates = null;
	this.dialogheaderState = null;
	this.descriptionState = null;
	this.tooltip = null;
	this.color = null;
	this.preference = null;
	this.type = null;
	this.value = null;
	this.descriptionPreference = null;
	this.add = null;
	this.update = null;
	this.tree = null;

	this.columns =
	{
		name: [0, 0], // colName: [index, order]
		type: [1, 0],
		value: [2, 0],
		description: [3, 0]
	};

	this.newSwitch = function()
	{
		var mySwitch = new Switch(null, "New Switch", "");
		if(this.renameSwitch(mySwitch))
		{
			var listitem = this.listboxSwitches.appendItem(mySwitch.getName());
			listitem.sw = mySwitch;
			this.listboxSwitches.selectedIndex = this.listboxSwitches.getRowCount() - 1;
		}
	};

	this.renameSwitch = function(mySwitch)
	{
		var refresh = false;
		
		if(mySwitch == null)
		{
			mySwitch = this.sw;
			refresh = true;
		}
		var name = prompt("Enter name of switch:", mySwitch.getName());
		if(name == null || name == "") return false;
		mySwitch.setName(name);

		if(refresh)
		{
			var listbox = this.listboxSwitches.getItemAtIndex(this.listboxSwitches.selectedIndex);
			listbox.setAttribute("label", listbox.sw.getName());
			this.refreshSwitches();
		}

		return true;
	};

	this.removeSwitch = function()
	{
		var selIndex = this.listboxSwitches.selectedIndex;
		if(selIndex < 0) return;
		this.listboxSwitches.removeItemAt(selIndex);
//		this.listboxSwitches.selectedIndex = selIndex-1;
		this.refreshSwitches();
	};

	this.newState = function()
	{
		var state = new State(null, "New State", "", "", "black");
		if(this.renameState(state))
		{
			this.sw.addState(state);
//			this.pr = new Preference("New Preference", BOOLEAN, "", "");
			var listitem = this.listboxStates.appendItem(state.getName());
			listitem.st = state;
			this.listboxStates.selectedIndex = this.listboxStates.getRowCount() - 1;
		}
	};

	this.renameState = function(state)
	{
		var refresh = false;

		if(state == null)
		{
			state = this.st;
			refresh = true;
		}
		var name = prompt("Enter name of state:", state.getName());
		if(name == null || name == "") return false;
		state.setName(name);

		if(refresh)
		{
			var listbox = this.listboxStates.getItemAtIndex(this.listboxStates.selectedIndex);
			listbox.setAttribute("label", listbox.st.getName());
			this.refreshStates();
		}

		return true;
	};

	this.removeState = function()
	{
		var selIndex = this.listboxStates.selectedIndex;
		if(selIndex < 0) return;
		this.listboxStates.removeItemAt(selIndex);
		this.sw.removeState(selIndex);
//		this.listboxStates.selectedIndex = selIndex-1;
		this.refreshStates();
	};

	this.removeAllStates = function()
	{
		this.sw.setStates([]);
		this.refreshSwitches();
	};

	this.removePreference = function()
	{
		var curIndex = this.tree.currentIndex;
		if(curIndex < 0) return;
		this.st.removePreference(curIndex);
		this.refreshStates();
	};

	this.removeAllPreferences = function()
	{
		this.st.setPreferences([]);
		this.refreshStates();
	};

	this.addPreference = function()
	{
		this.st.addPreference(this.pr);
		this.refreshStates();
	};

	this.updatePreference = function()
	{
		this.st.setPreference(this.index, this.pr);
		this.refreshStates();
	};

	this.load = function()
	{
		if("arguments" in window && window.arguments.length > 0) this.control = window.arguments[0];
		else this.control = new Control();
		this.listboxSwitches = $("listbox-switches");
		this.dialogheaderSwitch = $("dialogheader-switch");
		this.initial = $("initial");
		this.listboxStates = $("listbox-states");
		this.dialogheaderState = $("dialogheader-state");
		this.descriptionState = $("description-state");
		this.tooltip = $("tooltip");
		this.color = $("color-state");
		this.preference = $("preference");
		this.type = $("type");
		this.value = $("value-state");
		this.descriptionPreference = $("description-preference");
		this.add = $("add-preference");
		this.update = $("update-preference");
		this.tree = $("tree");
		this.listboxSwitches.addEventListener("select", this, false);
		this.listboxStates.addEventListener("select", this, false);
		this.tree.addEventListener("select", this, false);
		this.loadSwitches();
	};

	this.handleEvent = function(event)
	{
		switch(event.target.id)
		{
			case "listbox-switches": this.refreshSwitches(); break;
			case "listbox-states": this.refreshStates(); break;
			case "tree": this.refreshPreferences();
		}
	};

	this.loadSwitches = function()
	{
		while(this.listboxSwitches.getRowCount() > 0 && this.listboxSwitches.removeItemAt(0));
		var aSwitch = this.control.getSwitches();
		for(var i=0; i<aSwitch.length; i++)
		{
			var mySwitch = aSwitch[i];
			var listitem = this.listboxSwitches.appendItem(mySwitch.getName());
			listitem.sw = mySwitch;
		}
	};

	this.loadStates = function()
	{
		while(this.listboxStates.getRowCount() > 0 && this.listboxStates.removeItemAt(0));
		var aStates = this.sw.getStates();
		for(var i=0; i<aStates.length; i++)
		{
			var state = aStates[i];
			var listitem = this.listboxStates.appendItem(state.getName());
			listitem.st = state;
		}
	};

	this.loadPreferences = function()
	{
		if(this.st == null) return;
		var aPreferences = this.st.getPreferencesArray();
		this.tree.view = new TreeView(aPreferences, this.columns, aPreferences.length);
	};

	this.refreshSwitches = function()
	{
		if(this.listboxStates.selectedIndex < 0) this.refreshStates();
		else
		{
			this.listboxStates.selectedIndex = -1;/*this.refreshStates();*/
		}

		if(this.listboxSwitches.selectedIndex < 0)
		{
			this.dialogheaderSwitch.setAttribute("title", "");
			this.initial.value = "";
		}
		else
		{
			this.sw = this.listboxSwitches.getSelectedItem(0).sw;
			this.st = null;
			this.pr = null;
//			if(this.sw == null) return;
			this.dialogheaderSwitch.setAttribute("title", this.sw.getName());
			this.initial.value = this.sw.getInitial();

			this.loadStates();
			this.loadPreferences();
		}
	};

	this.refreshStates = function()
	{
		if(this.tree.currentIndex < 0) this.refreshPreferences();
		else {this.tree.currentIndex = -1;this.refreshPreferences();}

		if(this.listboxStates.selectedIndex < 0)
		{
			this.pr = null;
			this.dialogheaderState.setAttribute("title", "");
			this.descriptionState.value = "";
			this.tooltip.value = "";
			this.color.value = "";
		}
		else
		{
			this.st = this.listboxStates.getSelectedItem(0).st;
//			if(this.st == null) return;
			this.dialogheaderState.setAttribute("title", this.st.getName());
			this.descriptionState.value = this.st.getDescription();
			this.tooltip.value = this.st.getTooltip();
			this.color.value = this.st.getColor();
//			this.refreshPreferences();
			this.loadPreferences();
		}
	};

	this.refreshPreferences = function()
	{
		if(this.tree.currentIndex < 0)
		{
			this.pr = new Preference("", BOOLEAN, "", "");
			this.preference.value = this.pr.getName();
			this.type.selectedIndex = this.pr.getTypeInt();
			this.value.value = this.pr.getValue();
			this.descriptionPreference.value = this.pr.getDescription();
			this.tree.view = new TreeView([], this.columns, 0);
			this.update.hidden = true;
		}
		else
		{
			this.index = this.tree.currentIndex;
			this.pr = this.st.getPreference(this.index).createClone();
			this.preference.value = this.pr.getName();
			this.type.selectedIndex = this.pr.getTypeInt();
			this.value.value = this.pr.getValue();
			this.descriptionPreference.value = this.pr.getDescription();
			this.update.hidden = false;
		}
	};

	this.sortColumn = function(column)
	{
		var order = this.columns[column.id][1];
		this.columns[column.id][1] = order == 0 ? 1 : 0;
		this.tableSort(this.sw.getStates(), this.columns[column.id][0], order);
		this.tree.view = new TreeView(this.sw.getStates(), this.columns, this.sw.getStates().length);
	};

	this.tableSort = function(table, col, order)
	{
		var columnSort;
		if(order == 0) columnSort = function(a, b) { return a[col] < b[col] ? -1 : a[col] > b[col] ? 1 : 0; } // ascending
		else columnSort = function(a, b) { return a[col] > b[col] ? -1 : a[col] < b[col] ? 1 : 0; } // descending
		table.sort(columnSort);
	};

	this.accept = function()
	{
		var data = this.generateXml();
		this.control.saveSwitches(data);
		this.control.update(UPDATE_ALL);
	};

	this.generateXml = function()
	{
		var sResult = "" +
"<?xml version=\"1.0\" encoding=\"" + CHARSET + "\"?>\n" +
"<prefswitch>\n";
		for(var i=0; i<this.listboxSwitches.getRowCount(); i++)
		{
			var mySwitch = this.listboxSwitches.getItemAtIndex(i).sw;
			sResult += "" +
"	<switch id=\"" + mySwitch.getID() + "\" " +
		"name=\"" + this.control.encode(mySwitch.getName()) + "\" " +
		"initial=\"" + this.control.encode(mySwitch.getInitial()) + "\">\n";
			for(var j=0; j < mySwitch.getStates().length; j++)
			{
				var state = mySwitch.getState(j);
				sResult += "" +
"		<state id=\"" + state.getID() + "\" " +
			"name=\"" + this.control.encode(state.getName()) + "\" " +
			"description=\"" + this.control.encode(state.getDescription()) + "\" " +
			"tooltip=\"" + this.control.encode(state.getTooltip()) + "\" " +
			"color=\"" + this.control.encode(state.getColor()) + "\">\n";
				for(var k=0; k < state.getPreferences().length; k++)
				{
					var tPreference = state.getPreference(k);
					sResult += "" +
"			<preference name=\"" + this.control.encode(tPreference.getName()) + "\" " +
				"type=\"" + tPreference.getType() + "\" " +
				"value=\"" + this.control.encode("" + tPreference.getValue()) + "\" " +
				"description=\"" + this.control.encode(tPreference.getDescription()) + "\"/>\n";
				}
				sResult += "" +
"		</state>\n";
			}
			sResult += "" +
"	</switch>\n";
		}
		sResult += "" +
"</prefswitch>\n";
		return sResult;
	};
}

function TreeView(table, columns, rowcount)
{
	this.table = table;
	this.columns = columns;
	this.rowCount = rowcount;
//	this.cycleHeader = function(column) { sortColumn(column); };
	this.getCellProperties = function(row, col, props) {};
	this.getCellText = function(row, column) { return this.table[row][this.columns[column.id][0]]; };
	this.getColumnProperties = function(colid, col, props) {};
	this.getImageSrc = function(row, col) { return null; };
	this.getLevel = function(row) { return 0; };
	this.getRowProperties = function(row, props) {};
	this.isContainer = function(row) { return false; };
	this.isSeparator = function(row) { return false; };
	this.isSorted = function(row) { return false; };
	this.setTree = function(treebox) { this.treebox=treebox; };
}

function $(id) {return document.getElementById(id);}

var switchDialog = new SwitchDialog();
